package com.ibm.ift;

import java.time.Instant;
import java.util.List;

import com.ibm.broker.javacompute.MbJavaComputeNode;
import com.ibm.broker.plugin.MbElement;
import com.ibm.broker.plugin.MbException;
import com.ibm.broker.plugin.MbMessage;
import com.ibm.broker.plugin.MbMessageAssembly;
import com.ibm.broker.plugin.MbOutputTerminal;
import com.ibm.broker.plugin.MbUserException;

public class Authenticate_with_IFT_expiredtoken extends MbJavaComputeNode {

	public void evaluate(MbMessageAssembly inAssembly) throws MbException {
		MbOutputTerminal out = getOutputTerminal("out");
		MbOutputTerminal alt = getOutputTerminal("alternate");

		MbMessage inMessage = inAssembly.getMessage();
		MbMessageAssembly outAssembly = null;
		try {
			// create new message as a copy of the input
			MbMessage outMessage = new MbMessage(inMessage);
			outAssembly = new MbMessageAssembly(inAssembly, outMessage);
			// ----------------------------------------------------------
			// Add user code below

			//get the expired time from the token (if the token exists in the Local Environment)
			long exp_time = 0l;
			
			MbElement local_env = inAssembly.getLocalEnvironment().getRootElement();
			@SuppressWarnings("unchecked")
			List<MbElement> expir_obj = (List<MbElement>) local_env.evaluateXPath("(Variables/IFT/iam_token/expiration)[1]"); //XPath will return an array
					
			if (expir_obj != null) {
				exp_time = Long.parseLong(expir_obj.get(0).getValueAsString());
			}
						
			long curr_time = Instant.now().getEpochSecond();
						
			long diff = exp_time - curr_time; //assume that the exp_time is still in the future
			
			//If the diff is positive, then the token has expired.
			if (diff <= 0) {
				//propagate to the renew token flow
				alt.propagate(outAssembly);
				return;
			}
			//else propagate to the out terminal (see last line of code below)
			
			// End of user code
			// ----------------------------------------------------------
		} catch (MbException e) {
			// Re-throw to allow Broker handling of MbException
			throw e;
		} catch (RuntimeException e) {
			// Re-throw to allow Broker handling of RuntimeException
			throw e;
		} catch (Exception e) {
			// Consider replacing Exception with type(s) thrown by user code
			// Example handling ensures all exceptions are re-thrown to be handled in the flow
			throw new MbUserException(this, "evaluate()", "", "", e.toString(),
					null);
		}
		// The following should only be changed
		// if not propagating message to the 'out' terminal
		out.propagate(outAssembly);

	}

}
