package com.ibm.ift;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLEncoder;
import org.json.*;

import javax.net.ssl.HttpsURLConnection;

import com.ibm.broker.javacompute.MbJavaComputeNode;
import com.ibm.broker.plugin.MbElement;
import com.ibm.broker.plugin.MbException;
import com.ibm.broker.plugin.MbJSON;
import com.ibm.broker.plugin.MbMessage;
import com.ibm.broker.plugin.MbMessageAssembly;
import com.ibm.broker.plugin.MbOutputTerminal;
import com.ibm.broker.plugin.MbUserException;

public class Authenticate_get_IAM_Token extends MbJavaComputeNode {

	public void evaluate(MbMessageAssembly inAssembly) throws MbException {
		MbOutputTerminal out = getOutputTerminal("out");
		MbOutputTerminal alt = getOutputTerminal("alternate");

		MbMessage inMessage = inAssembly.getMessage();
		MbMessageAssembly outAssembly = null;
		try {
			// create new message as a copy of the input
			MbMessage outMessage = new MbMessage(inMessage);
			outAssembly = new MbMessageAssembly(inAssembly, outMessage);
			// ----------------------------------------------------------
			// Add user code below

			String requestURL = "https://iam.ng.bluemix.net/oidc/token";
			
			URL url;
	        String response = "";
	        try {
	            url = new URL(requestURL);

	            HttpURLConnection conn = (HttpURLConnection) url.openConnection();
	            conn.setReadTimeout(15000);
	            conn.setConnectTimeout(15000);
	            conn.setRequestMethod("POST");
	            conn.setDoInput(true);
	            conn.setDoOutput(true);


	            OutputStream os = conn.getOutputStream();
	            BufferedWriter writer = new BufferedWriter(
	                    new OutputStreamWriter(os, "UTF-8"));
	            
	            //writer.write(getPostDataString(postDataParams));
	            //String formData = "apikey=8rXjlQCBFuCyP0IhouDo6qEoXq39mhgGNyy7-B5ahSsv&grant_type=urn%3Aibm%3Aparams%3Aoauth%3Agrant-type%3Aapikey";
	            
	            String formData = "";
	            //append the grant_type
	            formData +=   URLEncoder.encode("grant_type", "UTF-8") + "=" 
	            		    + URLEncoder.encode("urn:ibm:params:oauth:grant-type:apikey", "UTF-8");
	            formData += "&" 
	            		    + URLEncoder.encode("apikey", "UTF-8") + "="
	            		    + URLEncoder.encode(inAssembly.getLocalEnvironment().getRootElement().getFirstElementByPath("/Variables/IFT/apikey").getValueAsString(), "UTF-8");
	            
	            writer.write(formData);

	            writer.flush();
	            writer.close();
	            os.close();
	            int responseCode=conn.getResponseCode();

	            if (responseCode == HttpsURLConnection.HTTP_OK) {
	                String line;
	                BufferedReader br=new BufferedReader(new InputStreamReader(conn.getInputStream()));
	                while ((line=br.readLine()) != null) {
	                    response+=line;
	                }

	                
	                /* The response from the previous node will look something like this:
	            		{"access_token":"eyJraWQiOiIyMDE3M...",
	            		"token_type":"Bearer",
	            		"expires_in":3600,
	            		"expiration":1529419825,
	            		"scope":"ibm openid"}
					*/
	        		MbElement outRoot = outMessage.getRootElement();
	        		MbElement outLocalEnv = outAssembly.getLocalEnvironment().getRootElement();
	        		
	        		//store the returned IAM token information so that we can check later to see if we need to renew before submitting data
	                JSONObject jsonResp = new JSONObject(response);

	        		MbElement outIamToken = outLocalEnv.getFirstElementByPath("Variables/IFT").createElementAsFirstChild(MbElement.TYPE_NAME_VALUE, "iam_token", null);
	        		
	        		//We will need this later...
	        		String access_token = jsonResp.get("access_token").toString();
	        		
	        		outIamToken.createElementAsLastChild(MbElement.TYPE_NAME_VALUE, "access_token", access_token);
	        		outIamToken.createElementAsLastChild(MbElement.TYPE_NAME_VALUE, "token_type", jsonResp.get("token_type").toString());
	        		outIamToken.createElementAsLastChild(MbElement.TYPE_NAME_VALUE, "expires_in", jsonResp.get("expires_in").toString());
	        		outIamToken.createElementAsLastChild(MbElement.TYPE_NAME_VALUE, "expiration", jsonResp.get("expiration").toString());
	        		outIamToken.createElementAsLastChild(MbElement.TYPE_NAME_VALUE, "scope", jsonResp.get("scope").toString());

	        		//Set the access_token as the payload for the following HTTPRequest Node
	        		MbElement outJsonRoot = outRoot.createElementAsLastChild(MbJSON.PARSER_NAME);
	        		MbElement outJsonData = outJsonRoot.createElementAsLastChild(MbElement.TYPE_NAME, MbJSON.DATA_ELEMENT_NAME, null);
	        		outJsonData.createElementAsLastChild(MbElement.TYPE_NAME_VALUE, "access_token", access_token);
	        		
	        		//set the content type
	        		MbElement httpReqHeader = outRoot.createElementAsLastChild(MbElement.TYPE_NAME_VALUE, "HTTPRequestHeader", null);
	        		httpReqHeader.createElementAsLastChild(MbElement.TYPE_NAME_VALUE, "Content-Type", "application/json");
	        		
	        		
	        		// assume production unless specified otherwise
	        		String base_url = "";
	        		
	        		String env = outLocalEnv.getFirstElementByPath("Variables/IFT/environment").getValueAsString(); 
	        		
	        		switch(env.toUpperCase()){
	        		case ("INTEGRATION"):
	        			base_url = "https://fs-identity-proxy-integration.mybluemix.net/exchange_token/v1/organization/";
	        			break;
	        		case ("DEV"):
        				base_url = "https://fs-identity-proxy-dev.mybluemix.net/exchange_token/v1/organization/";
	        			break;
	        		default: //production
	        			base_url = "https://fs-identity-proxy.mybluemix.net/exchange_token/v1/organization/";
	        		}
	        		
	        		
	        		//override the URL parameter to supply the data
	        		MbElement outDest = outLocalEnv.getFirstElementByPath("Destination");
	        		if (outDest == null) {
	        			outDest = outLocalEnv.createElementAsLastChild(MbElement.TYPE_NAME_VALUE, "Destination", null);
	        		}
	        		
	        		MbElement outHTTP = outDest.createElementAsLastChild(MbElement.TYPE_NAME_VALUE, "HTTP", null);
	        		outHTTP.createElementAsLastChild(MbElement.TYPE_NAME_VALUE, "RequestURL", base_url + outLocalEnv.getFirstElementByPath("Variables/IFT/orgId").getValueAsString());
	        		
	        		//SET OutputLocalEnvironment.Destination.HTTP.RequestURL = base_url || InputLocalEnvironment.Variables.IFT.orgId;
	        		
	            }
	            else {
	                response="";

	            }
	            
	            
	        } catch (Exception e) {
	            e.printStackTrace();
	        }
			
			
			
			// End of user code
			// ----------------------------------------------------------
		} catch (MbException e) {
			// Re-throw to allow Broker handling of MbException
			throw e;
		} catch (RuntimeException e) {
			// Re-throw to allow Broker handling of RuntimeException
			throw e;
		} catch (Exception e) {
			// Consider replacing Exception with type(s) thrown by user code
			// Example handling ensures all exceptions are re-thrown to be handled in the flow
			throw new MbUserException(this, "evaluate()", "", "", e.toString(),
					null);
		}
		// The following should only be changed
		// if not propagating message to the 'out' terminal
		out.propagate(outAssembly);

	}

}
